#include once "world.bi"


function world_struct.init( byref wWidth as integer, byref wHeight as integer ) as integer

	windowWidth = wWidth
	windowHeight = wHeight

	camera.zoom = 1

	particles.init(5000)

	SetConfigFlags( FLAG_WINDOW_RESIZABLE or FLAG_VSYNC_HINT )
	InitWindow( windowWidth, windowHeight, "DERP!" )

	screenBuffer = LoadRenderTexture( gameScreenWidth, gameScreenHeight )
	SetTextureFilter( screenBuffer.texture, FILTER_POINT )
	
	explosionTexture = LoadTexture("res/gfx/prt/smoke1.png")
	SetTextureFilter( explosionTexture, FILTER_POINT)
	
	fontTtf = LoadFontEx( "res/fnt/moon_of_jupiter.ttf", 12, 0, 250 )
	SetTextureFilter(fontTtf.texture, FILTER_POINT)
	
	SetWindowMinSize( 320, 240 )
	SetTargetFPS( 60 )

	redim grid(gridSizeX, gridSizeY)

	for i as integer = 0 to ubound(baseShape)

		for y as integer = 0 to 3

			for x as integer = 0 to 3

				read baseShape(i).block(x,y).ID


				select case as const i

					case 0
						baseShape(i).block(x,y).bclr = ORANGE
						
					case 1
						baseShape(i).block(x,y).bclr = YELLOW

					case 2
						baseShape(i).block(x,y).bclr = RAYBLUE

					case 3
						baseShape(i).block(x,y).bclr = RAYRED

					case 4
						baseShape(i).block(x,y).bclr = RAYGREEN

					case 5
						baseShape(i).block(x,y).bclr = PURPLE

					case 6
						baseShape(i).block(x,y).bclr = BROWN

				end select
				
				baseShape(i).block(x,y).fclr = DARKGRAY
				
			next

		next

	next

	redim soundIndex(5)

	return false

end function


function world_struct.terminate() as integer

	UnloadRenderTexture( screenBuffer )

	UnloadImage( colormapTexture )

	UnloadTexture( explosionTexture )

	'UnloadTexture( highlightTexture )

	soundSys.unload_all()

	CloseWindow()

	return true

end function


function world_struct.gen_random_part( isFirstPart as integer ) as integer

	dim as integer partID = int(rnd*(ubound(baseShape)+1))

	if isFirstPart then

		for y as integer = 0 to 3

			for x as integer = 0 to 3

				curShape.block(x,y) = baseShape(partID).block(x,y)

			next

		next

		curShape.shapeID = partID

		dim as integer tempID = int(rnd*(ubound(baseShape)+1))

		for y as integer = 0 to 3

			for x as integer = 0 to 3

				nextShape.block(x,y) = baseShape(partID).block(x,y)

			next

		next


		curShape.shapeID = partID
		nextShape.shapeID = tempID

	else

		curShape.copy( nextShape )

		for y as integer = 0 to 3

			for x as integer = 0 to 3

				nextShape.block(x,y) = baseShape(partID).block(x,y)

			next

		next

		nextShape.shapeID = partID

	end if

	return partID

end function


function world_struct.add_cur_piece_to_main( curPieceID as integer, curPosX as integer, curPosY as integer ) as integer

	for y as integer = 0 to 3

		for x as integer = 0 to 3

			if curShape.block(x,y).ID <> 0 then

				grid( x + curPosX, y + curPosY ) = curShape.block(x,y)

			end if

		next

	next

	return false

end function


function world_struct.new_level() as integer

	for y as integer = 0 to gridSizeY

		for x as integer = 0 to gridSizeX
			
			grid(x,y).ID = 0
			grid(x,y).fclr = color(64,64,64,255)

			if x = 0 or y = gridSizeY or x = gridSizeX then

				grid(x,y).ID = 1
				grid(x,y).fclr = color(64, 64, 64,255)
				grid(x,y).bclr = color(128, 128, 128,255)				
				
			end if

		next

	next

	return 0

end function


function world_struct.check_lines() as integer

	dim as integer lCount

	for y as integer = 0 to gridSizeY

		dim as integer bCount

		for x as integer = 0 to gridSizeX

			if grid(x,y).ID > 1 then

				bCount+=1

			end if

		next

		if bCount = gridSizeX-1 then
			
			destructMark(lCount) = y
			
			lCount+=1
			
		end if

	next

	return lCount

end function


function world_struct.remove_lines() as integer

	dim as integer lCount

	for y as integer = 0 to gridSizeY

		dim as integer bCount

		for x as integer = 0 to gridSizeX

			if grid(x,y).ID > 1 then

				bCount+=1

			end if

		next

		if bCount = gridSizeX-1 then

			lCount+=1

			for y2 as integer = y to 1 step -1

				for x as integer = 0 to gridSizeX

					grid(x,y2) = grid(x, y2-1)

				next

			next

		end if

	next

	return lCount

end function


function world_struct.check_left_move() as integer

	for y as integer = 0 to 3

		for x as integer = 0 to 3

			if curShape.block(x,y).ID = 0 then continue for

			if grid(x+curShapePosX-1, y+curshapePosY).ID <> 0 then

				return false

			end if

		next

	next

	return true

end function


function world_struct.check_right_move() as integer

	for y as integer = 0 to 3

		for x as integer = 0 to 3

			if curShape.block(x,y).ID = 0 then continue for

			if grid(x+curShapePosX+1, y+curshapePosY).ID <> 0 then

				return false

			end if

		next

	next
	
	return true
	
end function


function shape_struct.rotateCW() as integer

	static as shape_struct tShape

	for y as integer = 0 to 3

		for x as integer = 0 to 3

			tShape.block(y,3-x) = this.block(x,y)

		next

	next


	for y as integer = 0 to 3

		for x as integer = 0 to 3

			this.block(x,y) = tShape.block(x,y)

		next

	next

	return true

end function


function shape_struct.copy( oShape as shape_struct ) as integer

	for y as integer = 0 to 3

		for x as integer = 0 to 3

			this.block(x,y) = oShape.block(x,y)

		next

	next

	this.shapeID = oShape.shapeID

	return true

end function
